@@ -1,6 +1,6 @@
 #! /usr/bin/perl -w
 #
-# listadmin version 2.24
+# listadmin version 2.26
 # Written 2003 - 2005 by
 # Kjetil Torgrim Homme <kjetilho+listadmin@ifi.uio.no>
 # Released into public domain.
@@ -92,7 +92,9 @@ for my $list (@lists) {
     my %change = ();
 
     process_subscriptions ($list, $info, $config->{$list}, \%change);
-    approve_messages ($list, $info, $config->{$list}, \%change);
+    my $num = undef;
+ restart_approval:
+    approve_messages ($list, $info, $config->{$list}, \%change, $num);
 
     if ($config->{$list}->{"confirm"}) {
 	if (scalar %change) {
@@ -108,6 +110,9 @@ _END_
 	    if ($c =~ /^\s*(no?|nei|skip)\s*$/i) {
 		print "skipping ...\n";
 		next;
+	    } elsif ($c =~ /^\d$/) {
+		$num = $c - 1;
+		goto restart_approval;
 	    } elsif ($c !~ /^\s*(|ja?|y|yes)\s*$/i) {
 		goto redo_confirm;
 	    }
@@ -181,7 +186,7 @@ end
 }
 
 sub approve_messages {
-    my ($list, $info, $config, $change) = @_;
+    my ($list, $info, $config, $change, $num) = @_;
 
     my $listdef = $config->{"default"};
     my $spamlevel = $config->{"spamlevel"};
@@ -192,7 +197,13 @@ sub approve_messages {
     my $dis_reas = $config->{"discard_if_reason"};
 
     my $count = keys (%{$info}) - 1;	# subtract 1 for globals
-    my $num = 0;
+    my $search_pattern = "";
+    my $dont_skip_forward = 0;
+    if (!defined ($num)) {
+	$num = 0;
+    } else {
+	$dont_skip_forward = 1;
+    }
     my $prompt = 'Approve/Reject/Discard/Skip/view Body/view Full/jump #/Help/Quit';
     my @num_to_id = grep { ! /^global$/ } sort keys %{$info};
  msgloop:
@@ -220,7 +231,8 @@ sub approve_messages {
 		    if $dis_reas && got_match ($reason, $dis_reas);
 	    $ans ||= "d" if $match;
 	    $ans = undef if (($ns_subj && $subject =~ $ns_subj) ||
-			     ($ns_from && $from =~ $ns_from));
+			     ($ns_from && $from =~ $ns_from) ||
+			     $dont_skip_forward);
 
 	    if ($ans && $match) {
 		if ($match eq "spam") {
@@ -245,13 +257,43 @@ sub approve_messages {
 		last msgloop;
 	    } elsif ($ans eq "s") {
 		delete $change->{$id};
+		$dont_skip_forward = 0;
 		next msgloop;
 	    } elsif ($ans =~ /^\d+$/ && $ans > 0 && $ans <= $count) {
 		$num = $ans - 1;
+		$dont_skip_forward = 1;
 		next msgloop;
 	    } elsif ($ans eq "a" || $ans eq "d") {
 		$change->{$id} = [ $ans ];
+		$dont_skip_forward = 0;
 		last;
+	    } elsif ($ans =~ m,([/?])(.*),) {
+		my $i = $num - 1;
+		my $direction = 1;
+		my $fencepost = $count - 1;
+		if ($1 eq "?") {
+		    $direction = -1;
+		    $fencepost = 1;
+		}
+		# If no pattern is specified, reuse previous pattern.
+		$search_pattern = $2 unless $2 eq "";
+		if ($search_pattern eq "") {
+		    print "No search pattern specified.  Try 'help'\n";
+		    next;
+		}
+		while ($i != $fencepost) {
+		    $i += $direction;
+		    my $id = $num_to_id[$i];
+		    my $search_from = $info->{$id}{"from"};
+		    my $search_subject = $info->{$id}{"subject"} || "";
+		    if ($search_from =~ /$search_pattern/i ||
+			$search_subject =~ /$search_pattern/i) {
+			$num = $i;
+			$dont_skip_forward = 1;
+			next msgloop;
+		    }
+		}
+		print "Pattern not found\n"
 	    } elsif ($ans eq "r") {
 	    redo_reject:
 		my $def_reason = $info->{$id}{"rejreason"};
@@ -268,6 +310,7 @@ sub approve_messages {
 		}
 
 		$change->{$id} = [ "r", $r ];
+		$dont_skip_forward = 0;
 		last;
 	    } elsif ($ans eq "f") {
 		print $info->{$id}{"headers"}, "\n\n", $info->{$id}{"body"};
@@ -299,13 +342,15 @@ sub approve_messages {
 Choose one of the following actions by typing the corresponding letter
 and pressing Return.
 
-  a  Approve   -- the message will be sent to all member of the list
+  a  Approve   -- the message will be sent to all members of the list
   r  Reject    -- notify sender that the message was rejected
   d  Discard   -- throw message away, don't notify sender
   s  Skip      -- don't decide now, leave it for later
   b  view Body -- display the first 20 lines of the message
   f  view Full -- display the complete message, including headers
   #  jump      -- jump backward or forward to message number #
+  /pattern     -- search for next message with matching From or Subject
+  ?pattern     -- search for previous message with matching From or Subject
   q  Quit      -- go on to the next list
 
 end
@@ -332,32 +377,32 @@ sub mailman_params {
     return \%params;
 }
 
+sub uio_adminurl {
+    my ($domain) = @_;
+    return 'https://{domain}/mailman/{domain}/admindb/{list}'
+	    if ($domain eq 'lister.ping.uio.no');
+    return 'http://{domain}/mailman/admindb/{list}@{domain}'
+	    if ($domain eq "lister.uio.no");
+    return 'http://{subdomain}-lists.uio.no/mailman/admindb/{list}@{domain}'
+	    if ($domain =~ /^(\w+\.)?uio\.no$/);
+    undef;
+}
+
 sub mailman_url {
     my ($list, $pattern) = @_;
 
     my ($lp, $domain) = split ('@', $list);
-    if ($pattern) {
-	my $url = $pattern;
-	my $subdom = $domain;
-	$subdom = $` if $subdom =~ /\./;
-	$url =~ s/\{list\}/$lp/g;
-	$url =~ s/\{domain\}/$domain/g;
-	$url =~ s/\{subdomain\}/$subdom/g;
-	return $url;
-    }
 
-    my $www = $domain;
-    if ($domain eq "lister.ping.uio.no") {
-	return "https://$domain/mailman/$domain/admindb/$lp";
-    } elsif ($domain =~ /^(\w+)\.uio\.no$/) {
-	$www = "$1-lists.uio.no";
-    } elsif ($domain eq "uio.no") {
-	$www = "uio-lists.uio.no";
-    } elsif ($hostname =~ /uio.no$/) {
-	# horrific.  this default should be split into a site specific file.
-	$www = "lister.uio.no";
-    }
-    return "http://$www/mailman/admindb/$list";
+    $pattern ||= uio_adminurl ($domain);
+    $pattern ||= 'http://{domain}/mailman/admindb/{list}';
+
+    my $url = $pattern;
+    my $subdom = $domain;
+    $subdom = $` if $subdom =~ /\./;
+    $url =~ s/\{list\}/$lp/g;
+    $url =~ s/\{domain\}/$domain/g;
+    $url =~ s/\{subdomain\}/$subdom/g;
+    return $url;
 }
 
 sub get_list {
